html{
    height: 100%;
}

body{
    display: flex;
    flex-direction: column;
    height: 100%;
    min-height: 100vh;
}

.container {
    display: flex;
    flex-direction: column;
    height: 100%;
}

header{
   /* 0 flex-grow, 0 flex-shrink, auto flex-basis */
   flex: 0 0 auto;
}
.site-content{
   /* 1 flex-grow, 0 flex-shrink, auto flex-basis */
   flex: 1 0 auto;
}
footer{
   /* 0 flex-grow, 0 flex-shrink, auto flex-basis */
   flex: 0 0 auto;
}
