#Json protocol top level keys
TYPE = 'type'
PAYLOAD = 'payload'
ID = 'msg_id'
REPLY_ID = 'reply_id'
#Json protocol type field
TYPE_ERROR = 'error'
TYPE_TASK = 'task'
TYPE_SOLUTION = 'solution'
TYPE_ACK = 'ack'
TYPE_UPDATE = 'update'
TYPE_STOP = 'stop'

#Error reply fields
ERROR_CODE = 'code'
ERROR_TITLE = 'title'
ERROR_DESC = 'description'